@@ -1,6 +1,9 @@
-use codec::Encode;
 use std::io::Write;
 
+use codec::Encode;
+
+#[cfg(feature = "arbitrary")]
+pub mod arbitrary;
 pub mod codec;
 pub mod parse;
 pub mod state;
