@@ -1,3 +1,16 @@
+use std::convert::{TryFrom, TryInto};
+
+use abnf_core::streaming::{CRLF_relaxed as CRLF, SP};
+use base64::decode as b64decode;
+use nom::{
+    branch::alt,
+    bytes::streaming::{tag, tag_no_case},
+    combinator::{map, map_opt, map_res, opt, value},
+    multi::{many1, separated_list0, separated_list1},
+    sequence::{delimited, preceded, terminated, tuple},
+    IResult,
+};
+
 use crate::{
     parse::{
         algorithm, auth_type,
@@ -12,21 +25,12 @@ use crate::{
     },
     types::{
         command::{Command, CommandBody, SearchKey},
-        core::astr,
-        data_items::{DataItem, Macro, MacroOrDataItems},
+        core::{astr, NonEmptyVec, NonZeroBytes},
+        fetch_attributes::{FetchAttribute, Macro, MacroOrFetchAttributes},
         flag::{Flag, StoreResponse, StoreType},
         AuthMechanism,
     },
 };
-use abnf_core::streaming::{CRLF_relaxed as CRLF, SP};
-use nom::{
-    branch::alt,
-    bytes::streaming::{tag, tag_no_case},
-    combinator::{map, map_opt, map_res, opt, value},
-    multi::{many1, separated_list0, separated_list1},
-    sequence::{delimited, preceded, terminated, tuple},
-    IResult,
-};
 
 /// command = tag SP (command-any /
 ///                   command-auth /
@@ -111,7 +115,7 @@ fn append(input: &[u8]) -> IResult<&[u8], CommandBody> {
             mailbox,
             flags: flags.unwrap_or_default(),
             date: date_time,
-            message: literal.to_vec(),
+            message: NonZeroBytes::try_from(literal).unwrap(), // Safe to unwrap
         },
     ))
 }
@@ -213,9 +217,15 @@ fn status(input: &[u8]) -> IResult<&[u8], CommandBody> {
         delimited(tag(b"("), separated_list0(SP, status_att), tag(b")")),
     ));
 
-    let (remaining, (_, _, mailbox, _, items)) = parser(input)?;
+    let (remaining, (_, _, mailbox, _, attributes)) = parser(input)?;
 
-    Ok((remaining, CommandBody::Status { mailbox, items }))
+    Ok((
+        remaining,
+        CommandBody::Status {
+            mailbox,
+            attributes,
+        },
+    ))
 }
 
 /// subscribe = "SUBSCRIBE" SP mailbox
@@ -252,7 +262,12 @@ fn enable(input: &[u8]) -> IResult<&[u8], CommandBody> {
 
     let (remaining, (_, capabilities)) = parser(input)?;
 
-    Ok((remaining, CommandBody::Enable { capabilities }))
+    Ok((
+        remaining,
+        CommandBody::Enable {
+            capabilities: capabilities.try_into().unwrap(),
+        },
+    ))
 }
 
 /// compress = "COMPRESS" SP algorithm
@@ -287,9 +302,11 @@ pub fn idle_done(input: &[u8]) -> IResult<&[u8], ()> {
 fn command_nonauth(input: &[u8]) -> IResult<&[u8], CommandBody> {
     let mut parser = alt((
         login,
-        map(authenticate, |(mechanism, ir)| CommandBody::Authenticate {
-            mechanism,
-            initial_response: ir.map(|i| i.to_owned()),
+        map(authenticate, |(mechanism, initial_response)| {
+            CommandBody::Authenticate {
+                mechanism,
+                initial_response,
+            }
         }),
         value(CommandBody::StartTLS, tag_no_case(b"STARTTLS")),
     ));
@@ -335,7 +352,7 @@ fn password(input: &[u8]) -> IResult<&[u8], astr> {
 ///                                            |
 ///                                            Added by SASL-IR (RFC RFC 4959)
 /// ```
-fn authenticate(input: &[u8]) -> IResult<&[u8], (AuthMechanism, Option<&str>)> {
+fn authenticate(input: &[u8]) -> IResult<&[u8], (AuthMechanism, Option<Vec<u8>>)> {
     let mut parser = tuple((
         tag_no_case(b"AUTHENTICATE"),
         SP,
@@ -346,11 +363,24 @@ fn authenticate(input: &[u8]) -> IResult<&[u8], (AuthMechanism, Option<&str>)> {
         )),
     ));
 
-    let (remaining, (_, _, auth_type, ir)) = parser(input)?;
+    let (remaining, (_, _, auth_type, base64)) = parser(input)?;
 
     // Server must send continuation ("+ ") at this point...
 
-    Ok((remaining, (auth_type, ir)))
+    match base64 {
+        Some(base64) => {
+            match b64decode(base64) {
+                Ok(ir) => Ok((remaining, (auth_type, Some(ir)))),
+                Err(_) => {
+                    return Err(nom::Err::Error(nom::error::make_error(
+                        input,
+                        nom::error::ErrorKind::Verify, // TODO(verify): use `Failure` or `Error`?
+                    )));
+                }
+            }
+        }
+        None => Ok((remaining, (auth_type, None))),
+    }
 }
 
 /// Use this parser instead of command when doing authentication.
@@ -418,26 +448,35 @@ fn fetch(input: &[u8]) -> IResult<&[u8], CommandBody> {
         sequence_set,
         SP,
         alt((
-            value(MacroOrDataItems::Macro(Macro::All), tag_no_case(b"ALL")),
-            value(MacroOrDataItems::Macro(Macro::Fast), tag_no_case(b"FAST")),
-            value(MacroOrDataItems::Macro(Macro::Full), tag_no_case(b"FULL")),
+            value(
+                MacroOrFetchAttributes::Macro(Macro::All),
+                tag_no_case(b"ALL"),
+            ),
+            value(
+                MacroOrFetchAttributes::Macro(Macro::Fast),
+                tag_no_case(b"FAST"),
+            ),
+            value(
+                MacroOrFetchAttributes::Macro(Macro::Full),
+                tag_no_case(b"FULL"),
+            ),
             map(fetch_att, |fetch_att| {
-                MacroOrDataItems::DataItems(vec![fetch_att])
+                MacroOrFetchAttributes::FetchAttributes(vec![fetch_att])
             }),
             map(
                 delimited(tag(b"("), separated_list0(SP, fetch_att), tag(b")")),
-                MacroOrDataItems::DataItems,
+                MacroOrFetchAttributes::FetchAttributes,
             ),
         )),
     ));
 
-    let (remaining, (_, _, sequence_set, _, items)) = parser(input)?;
+    let (remaining, (_, _, sequence_set, _, attributes)) = parser(input)?;
 
     Ok((
         remaining,
         CommandBody::Fetch {
             sequence_set,
-            items,
+            attributes,
             uid: false,
         },
     ))
@@ -451,12 +490,12 @@ fn fetch(input: &[u8]) -> IResult<&[u8], CommandBody> {
 ///             "UID" /
 ///             "BODY" section ["<" number "." nz-number ">"] /
 ///             "BODY.PEEK" section ["<" number "." nz-number ">"]
-fn fetch_att(input: &[u8]) -> IResult<&[u8], DataItem> {
+fn fetch_att(input: &[u8]) -> IResult<&[u8], FetchAttribute> {
     alt((
-        value(DataItem::Envelope, tag_no_case(b"ENVELOPE")),
-        value(DataItem::Flags, tag_no_case(b"FLAGS")),
-        value(DataItem::InternalDate, tag_no_case(b"INTERNALDATE")),
-        value(DataItem::BodyStructure, tag_no_case(b"BODYSTRUCTURE")),
+        value(FetchAttribute::Envelope, tag_no_case(b"ENVELOPE")),
+        value(FetchAttribute::Flags, tag_no_case(b"FLAGS")),
+        value(FetchAttribute::InternalDate, tag_no_case(b"INTERNALDATE")),
+        value(FetchAttribute::BodyStructure, tag_no_case(b"BODYSTRUCTURE")),
         map(
             tuple((
                 tag_no_case(b"BODY.PEEK"),
@@ -467,7 +506,7 @@ fn fetch_att(input: &[u8]) -> IResult<&[u8], DataItem> {
                     tag(b">"),
                 )),
             )),
-            |(_, section, byterange)| DataItem::BodyExt {
+            |(_, section, byterange)| FetchAttribute::BodyExt {
                 section,
                 partial: byterange.map(|(start, _, end)| (start, end)),
                 peek: true,
@@ -483,17 +522,18 @@ fn fetch_att(input: &[u8]) -> IResult<&[u8], DataItem> {
                     tag(b">"),
                 )),
             )),
-            |(_, section, byterange)| DataItem::BodyExt {
+            |(_, section, byterange)| FetchAttribute::BodyExt {
                 section,
                 partial: byterange.map(|(start, _, end)| (start, end)),
                 peek: false,
             },
         ),
-        value(DataItem::Body, tag_no_case(b"BODY")),
-        value(DataItem::Uid, tag_no_case(b"UID")),
-        value(DataItem::Rfc822Header, tag_no_case(b"RFC822.HEADER")),
-        value(DataItem::Rfc822Size, tag_no_case(b"RFC822.SIZE")),
-        value(DataItem::Rfc822Text, tag_no_case(b"RFC822.TEXT")),
+        value(FetchAttribute::Body, tag_no_case(b"BODY")),
+        value(FetchAttribute::Uid, tag_no_case(b"UID")),
+        value(FetchAttribute::Rfc822Header, tag_no_case(b"RFC822.HEADER")),
+        value(FetchAttribute::Rfc822Size, tag_no_case(b"RFC822.SIZE")),
+        value(FetchAttribute::Rfc822Text, tag_no_case(b"RFC822.TEXT")),
+        value(FetchAttribute::Rfc822, tag_no_case(b"RFC822")),
     ))(input)
 }
 
@@ -583,7 +623,7 @@ fn search(input: &[u8]) -> IResult<&[u8], CommandBody> {
     let criteria = match criteria.len() {
         0 => unreachable!(),
         1 => criteria[0].clone(),
-        _ => SearchKey::And(criteria),
+        _ => SearchKey::And(NonEmptyVec::try_from(criteria).unwrap()), // Safe to unwrap
     };
 
     Ok((
@@ -762,7 +802,7 @@ fn search_key_limited<'a>(
                 |val| match val.len() {
                     0 => unreachable!(),
                     1 => val[0].clone(),
-                    _ => SearchKey::And(val),
+                    _ => SearchKey::And(NonEmptyVec::try_from(val).unwrap()), // Safe to unwrap
                 },
             ),
         )),
@@ -771,12 +811,14 @@ fn search_key_limited<'a>(
 
 #[cfg(test)]
 mod test {
+    use std::{convert::TryInto, num::NonZeroU32};
+
     use super::*;
     use crate::types::{
+        fetch_attributes::Section,
         response::Capability,
-        sequence::{SeqNo, Sequence},
+        sequence::{SeqNo, Sequence, SequenceSet as SequenceSetData},
     };
-    use std::convert::TryInto;
 
     #[test]
     fn test_fetch() {
@@ -786,6 +828,63 @@ mod test {
         println!("{:#?}", fetch(b"fetch 1:1 (flags)???"));
     }
 
+    #[test]
+    fn test_fetch_att() {
+        let tests = [
+            (FetchAttribute::Envelope, "ENVELOPE???"),
+            (FetchAttribute::Flags, "FLAGS???"),
+            (FetchAttribute::InternalDate, "INTERNALDATE???"),
+            (FetchAttribute::Rfc822, "RFC822???"),
+            (FetchAttribute::Rfc822Header, "RFC822.HEADER???"),
+            (FetchAttribute::Rfc822Size, "RFC822.SIZE???"),
+            (FetchAttribute::Rfc822Text, "RFC822.TEXT???"),
+            (FetchAttribute::Body, "BODY???"),
+            (FetchAttribute::BodyStructure, "BODYSTRUCTURE???"),
+            (FetchAttribute::Uid, "UID???"),
+            (
+                FetchAttribute::BodyExt {
+                    partial: None,
+                    peek: false,
+                    section: None,
+                },
+                "BODY[]???",
+            ),
+            (
+                FetchAttribute::BodyExt {
+                    partial: None,
+                    peek: true,
+                    section: None,
+                },
+                "BODY.PEEK[]???",
+            ),
+            (
+                FetchAttribute::BodyExt {
+                    partial: None,
+                    peek: true,
+                    section: Some(Section::Text(None)),
+                },
+                "BODY.PEEK[TEXT]???",
+            ),
+            (
+                FetchAttribute::BodyExt {
+                    partial: Some((42, NonZeroU32::try_from(1337).unwrap())),
+                    peek: true,
+                    section: Some(Section::Text(None)),
+                },
+                "BODY.PEEK[TEXT]<42.1337>???",
+            ),
+        ];
+
+        let expected_remainder = "???".as_bytes();
+
+        for (expected, test) in tests {
+            let (got_remainder, got) = fetch_att(test.as_bytes()).unwrap();
+
+            assert_eq!(expected, got);
+            assert_eq!(expected_remainder, got_remainder);
+        }
+    }
+
     #[test]
     fn test_search() {
         use SearchKey::*;
@@ -797,7 +896,7 @@ mod test {
             val,
             CommandBody::Search {
                 charset: None,
-                criteria: Uid(vec![Single(Value(5))]),
+                criteria: Uid(SequenceSetData(vec![Single(Value(5.try_into().unwrap()))])),
                 uid: false,
             }
         );
@@ -806,16 +905,24 @@ mod test {
         let expected = CommandBody::Search {
             charset: None,
             criteria: And(vec![
-                Uid(vec![Single(Value(5))]),
+                Uid(SequenceSetData(vec![Single(Value(5.try_into().unwrap()))])),
                 Or(
-                    Box::new(Uid(vec![Single(Value(5))])),
+                    Box::new(Uid(SequenceSetData(vec![Single(Value(
+                        5.try_into().unwrap(),
+                    ))]))),
                     Box::new(And(vec![
-                        Uid(vec![Single(Value(1))]),
-                        Uid(vec![Single(Value(2))]),
-                    ])),
+                        Uid(SequenceSetData(vec![Single(Value(1.try_into().unwrap()))])),
+                        Uid(SequenceSetData(vec![Single(Value(2.try_into().unwrap()))])),
+                    ]
+                    .try_into()
+                    .unwrap())),
                 ),
-                Not(Box::new(Uid(vec![Single(Value(5))]))),
-            ]),
+                Not(Box::new(Uid(SequenceSetData(vec![Single(Value(
+                    5.try_into().unwrap(),
+                ))])))),
+            ]
+            .try_into()
+            .unwrap()),
             uid: false,
         };
         assert_eq!(val, expected);
@@ -840,6 +947,8 @@ mod test {
                         Capability::Other("UTF8=ACCEPT".try_into().unwrap()),
                         Capability::Enable
                     ]
+                    .try_into()
+                    .unwrap()
                 }
             ),
             got
