@@ -2,24 +2,30 @@
 //!
 //! see https://tools.ietf.org/html/rfc3501#section-6
 
+use std::{convert::TryInto, io::Write};
+
+#[cfg(feature = "arbitrary")]
+use arbitrary::Arbitrary;
+use base64::encode as b64encode;
+#[cfg(feature = "serdex")]
+use serde::{Deserialize, Serialize};
+
 use crate::{
     codec::Encode,
     types::{
-        core::{AString, Atom, Charset, Tag},
-        data_items::MacroOrDataItems,
+        core::{AString, Atom, Charset, NonEmptyVec, NonZeroBytes, Tag},
+        datetime::{MyDateTime, MyNaiveDate},
+        fetch_attributes::MacroOrFetchAttributes,
         flag::{Flag, StoreResponse, StoreType},
         mailbox::{ListMailbox, Mailbox},
         response::Capability,
-        sequence::{Sequence, ToSequence},
+        sequence::SequenceSet,
         AuthMechanism, CompressionAlgorithm,
     },
     utils::{gen_tag, join_serializable},
 };
-use chrono::{DateTime, FixedOffset, NaiveDate};
-#[cfg(feature = "serdex")]
-use serde::{Deserialize, Serialize};
-use std::io::Write;
 
+#[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
 #[cfg_attr(feature = "serdex", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct Command {
@@ -48,111 +54,173 @@ impl Command {
         Command::new(gen_tag(), CommandBody::StartTLS)
     }
 
-    pub fn authenticate(mechanism: AuthMechanism, initial_response: Option<&str>) -> Command {
+    pub fn authenticate(mechanism: AuthMechanism, initial_response: Option<&[u8]>) -> Command {
         Command::new(
             gen_tag(),
             CommandBody::Authenticate {
                 mechanism,
-                initial_response: initial_response.map(|str| str.to_string()),
+                initial_response: initial_response.map(|bytes| bytes.to_vec()),
             },
         )
     }
 
-    pub fn login<U: Into<AString>, P: Into<AString>>(username: U, password: P) -> Command {
-        Command::new(
+    pub fn login<U: TryInto<AString>, P: TryInto<AString>>(
+        username: U,
+        password: P,
+    ) -> Result<Command, ()> {
+        Ok(Command::new(
             gen_tag(),
             CommandBody::Login {
-                username: username.into(), // FIXME(misuse): this should be TryInto. Fix in AString
-                password: password.into(), // FIXME(misuse): this should be TryInto. Fix in AString
+                username: username.try_into().map_err(|_| ())?,
+                password: password.try_into().map_err(|_| ())?,
             },
-        )
+        ))
     }
 
-    pub fn select(mailbox: Mailbox) -> Command {
-        Command::new(gen_tag(), CommandBody::Select { mailbox })
+    pub fn select<M>(mailbox: M) -> Result<Command, M::Error>
+    where
+        M: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
+            gen_tag(),
+            CommandBody::Select {
+                mailbox: mailbox.try_into()?,
+            },
+        ))
     }
 
-    pub fn examine(mailbox: Mailbox) -> Command {
-        Command::new(gen_tag(), CommandBody::Examine { mailbox })
+    pub fn examine<M>(mailbox: M) -> Result<Command, M::Error>
+    where
+        M: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
+            gen_tag(),
+            CommandBody::Examine {
+                mailbox: mailbox.try_into()?,
+            },
+        ))
     }
 
-    pub fn create(mailbox: Mailbox) -> Command {
-        Command::new(gen_tag(), CommandBody::Create { mailbox })
+    pub fn create<M>(mailbox: M) -> Result<Command, M::Error>
+    where
+        M: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
+            gen_tag(),
+            CommandBody::Create {
+                mailbox: mailbox.try_into()?,
+            },
+        ))
     }
 
-    pub fn delete(mailbox: Mailbox) -> Command {
-        Command::new(gen_tag(), CommandBody::Delete { mailbox })
+    pub fn delete<M>(mailbox: M) -> Result<Command, M::Error>
+    where
+        M: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
+            gen_tag(),
+            CommandBody::Delete {
+                mailbox: mailbox.try_into()?,
+            },
+        ))
     }
 
-    pub fn rename(mailbox: Mailbox, new_mailbox: Mailbox) -> Command {
-        Command::new(
+    pub fn rename<M1, M2>(mailbox: M1, new_mailbox: M2) -> Result<Command, ()>
+    where
+        M1: TryInto<Mailbox>,
+        M2: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
             gen_tag(),
             CommandBody::Rename {
-                mailbox,
-                new_mailbox,
+                mailbox: mailbox.try_into().map_err(|_| ())?,
+                new_mailbox: new_mailbox.try_into().map_err(|_| ())?,
             },
-        )
+        ))
     }
 
-    pub fn subscribe(mailbox: Mailbox) -> Command {
-        Command::new(gen_tag(), CommandBody::Subscribe { mailbox })
+    pub fn subscribe<M>(mailbox: M) -> Result<Command, M::Error>
+    where
+        M: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
+            gen_tag(),
+            CommandBody::Subscribe {
+                mailbox: mailbox.try_into()?,
+            },
+        ))
     }
 
-    pub fn unsubscribe(mailbox: Mailbox) -> Command {
-        Command::new(gen_tag(), CommandBody::Unsubscribe { mailbox })
+    pub fn unsubscribe<M>(mailbox: M) -> Result<Command, M::Error>
+    where
+        M: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
+            gen_tag(),
+            CommandBody::Unsubscribe {
+                mailbox: mailbox.try_into()?,
+            },
+        ))
     }
 
-    pub fn list<A: Into<Mailbox>, B: Into<ListMailbox>>(
+    pub fn list<A: TryInto<Mailbox>, B: TryInto<ListMailbox>>(
         reference: A,
         mailbox_wildcard: B,
-    ) -> Command {
-        Command::new(
+    ) -> Result<Command, ()> {
+        Ok(Command::new(
             gen_tag(),
             CommandBody::List {
-                reference: reference.into(),
-                mailbox_wildcard: mailbox_wildcard.into(),
+                reference: reference.try_into().map_err(|_| ())?,
+                mailbox_wildcard: mailbox_wildcard.try_into().map_err(|_| ())?,
             },
-        )
+        ))
     }
 
-    pub fn lsub<A: Into<Mailbox>, B: Into<ListMailbox>>(
+    pub fn lsub<A: TryInto<Mailbox>, B: TryInto<ListMailbox>>(
         reference: A,
         mailbox_wildcard: B,
-    ) -> Command {
-        Command::new(
+    ) -> Result<Command, ()> {
+        Ok(Command::new(
             gen_tag(),
             CommandBody::Lsub {
-                reference: reference.into(),
-                mailbox_wildcard: mailbox_wildcard.into(),
+                reference: reference.try_into().map_err(|_| ())?,
+                mailbox_wildcard: mailbox_wildcard.try_into().map_err(|_| ())?,
             },
-        )
+        ))
     }
 
-    pub fn status<M: Into<Mailbox>>(mailbox: M, items: Vec<StatusItem>) -> Command {
-        Command::new(
+    pub fn status<M>(mailbox: M, attributes: Vec<StatusAttribute>) -> Result<Command, M::Error>
+    where
+        M: TryInto<Mailbox>,
+    {
+        Ok(Command::new(
             gen_tag(),
             CommandBody::Status {
-                mailbox: mailbox.into(),
-                items,
+                mailbox: mailbox.try_into()?,
+                attributes,
             },
-        )
+        ))
     }
 
-    pub fn append<M: Into<Mailbox>>(
+    pub fn append<M, D>(
         mailbox: M,
         flags: Vec<Flag>,
-        date: Option<DateTime<FixedOffset>>,
-        message: Vec<u8>,
-    ) -> Command {
-        Command::new(
+        date: Option<MyDateTime>,
+        message: D,
+    ) -> Result<Command, ()>
+    where
+        M: TryInto<Mailbox>,
+        D: TryInto<NonZeroBytes>,
+    {
+        Ok(Command::new(
             gen_tag(),
             CommandBody::Append {
-                mailbox: mailbox.into(),
+                mailbox: mailbox.try_into().map_err(|_| ())?,
                 flags,
                 date,
-                message,
+                message: message.try_into().map_err(|_| ())?,
             },
-        )
+        ))
     }
 
     pub fn check() -> Command {
@@ -167,29 +235,32 @@ impl Command {
         Command::new(gen_tag(), CommandBody::Expunge)
     }
 
-    pub fn search(charset: Option<String>, criteria: SearchKey, uid: bool) -> Command {
-        Command::new(
+    pub fn search<C>(charset: C, criteria: SearchKey, uid: bool) -> Result<Command, C::Error>
+    where
+        C: TryInto<Option<Charset>>,
+    {
+        Ok(Command::new(
             gen_tag(),
             CommandBody::Search {
-                charset: charset.map(Charset),
+                charset: charset.try_into()?,
                 criteria,
                 uid,
             },
-        )
+        ))
     }
 
-    pub fn fetch<S, I>(sequence_set: S, items: I, uid: bool) -> Result<Command, ()>
+    pub fn fetch<S, I>(sequence_set: S, attributes: I, uid: bool) -> Result<Command, S::Error>
     where
-        S: ToSequence,
-        I: Into<MacroOrDataItems>,
+        S: TryInto<SequenceSet>,
+        I: Into<MacroOrFetchAttributes>,
     {
-        let sequence_set = sequence_set.to_sequence()?;
+        let sequence_set = sequence_set.try_into()?;
 
         Ok(Command::new(
             gen_tag(),
             CommandBody::Fetch {
                 sequence_set,
-                items: items.into(),
+                attributes: attributes.into(),
                 uid,
             },
         ))
@@ -201,11 +272,11 @@ impl Command {
         response: StoreResponse,
         flags: Vec<Flag>,
         uid: bool,
-    ) -> Result<Command, ()>
+    ) -> Result<Command, S::Error>
     where
-        S: ToSequence,
+        S: TryInto<SequenceSet>,
     {
-        let sequence_set = sequence_set.to_sequence()?;
+        let sequence_set = sequence_set.try_into()?;
 
         Ok(Command::new(
             gen_tag(),
@@ -221,16 +292,14 @@ impl Command {
 
     pub fn copy<S, M>(sequence_set: S, mailbox: M, uid: bool) -> Result<Command, ()>
     where
-        S: ToSequence,
-        M: Into<Mailbox>,
+        S: TryInto<SequenceSet>,
+        M: TryInto<Mailbox>,
     {
-        let sequence_set = sequence_set.to_sequence()?;
-
         Ok(Command::new(
             gen_tag(),
             CommandBody::Copy {
-                sequence_set,
-                mailbox: mailbox.into(),
+                sequence_set: sequence_set.try_into().map_err(|_| ())?,
+                mailbox: mailbox.try_into().map_err(|_| ())?,
                 uid,
             },
         ))
@@ -240,8 +309,16 @@ impl Command {
         Command::new(gen_tag(), CommandBody::Idle)
     }
 
-    pub fn enable(capabilities: Vec<Capability>) -> Command {
-        Command::new(gen_tag(), CommandBody::Enable { capabilities })
+    pub fn enable<C>(capabilities: C) -> Result<Command, C::Error>
+    where
+        C: TryInto<NonEmptyVec<Capability>>,
+    {
+        Ok(Command::new(
+            gen_tag(),
+            CommandBody::Enable {
+                capabilities: capabilities.try_into()?,
+            },
+        ))
     }
 
     pub fn name(&self) -> &'static str {
@@ -258,6 +335,7 @@ impl Encode for Command {
     }
 }
 
+#[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
 #[cfg_attr(feature = "serdex", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum CommandBody {
@@ -448,7 +526,8 @@ pub enum CommandBody {
     /// the user name whose privileges the client is requesting.
     Authenticate {
         mechanism: AuthMechanism,
-        initial_response: Option<String>,
+        /// Already base64-decoded
+        initial_response: Option<Vec<u8>>,
     },
 
     /// ### 6.2.3.  LOGIN Command
@@ -971,11 +1050,9 @@ pub enum CommandBody {
     ///   in its results, clients SHOULD NOT expect to be able to
     ///   issue many consecutive STATUS commands and obtain
     ///   reasonable performance.
-    ///
-    /// See [StatusDataItem](StatusDataItem).
     Status {
         mailbox: Mailbox,
-        items: Vec<StatusItem>,
+        attributes: Vec<StatusAttribute>,
     },
 
     /// 6.3.11. APPEND Command
@@ -1037,8 +1114,8 @@ pub enum CommandBody {
     Append {
         mailbox: Mailbox,
         flags: Vec<Flag>,
-        date: Option<DateTime<FixedOffset>>,
-        message: Vec<u8>,
+        date: Option<MyDateTime>,
+        message: NonZeroBytes,
     },
 
     // ----- Selected State (https://tools.ietf.org/html/rfc3501#section-6.4) -----
@@ -1191,11 +1268,9 @@ pub enum CommandBody {
     ///   For example, if a client receives an ENVELOPE for a
     ///   message when it already knows the envelope, it can
     ///   safely ignore the newly transmitted envelope.
-    ///
-    /// See [DataItem](../data_items/index.html) for more information.
     Fetch {
-        sequence_set: Vec<Sequence>,
-        items: MacroOrDataItems,
+        sequence_set: SequenceSet,
+        attributes: MacroOrFetchAttributes,
         uid: bool,
     },
 
@@ -1250,10 +1325,10 @@ pub enum CommandBody {
     /// -FLAGS.SILENT <flag list>
     ///    Equivalent to -FLAGS, but without returning a new value.
     Store {
-        sequence_set: Vec<Sequence>,
+        sequence_set: SequenceSet,
         kind: StoreType,
         response: StoreResponse,
-        flags: Vec<Flag>,
+        flags: Vec<Flag>, // FIXME(misuse): must not accept "\*" or "\Recent"
         uid: bool,
     },
 
@@ -1284,7 +1359,7 @@ pub enum CommandBody {
     /// implementations MUST restore the destination mailbox to its state
     /// before the COPY attempt.
     Copy {
-        sequence_set: Vec<Sequence>,
+        sequence_set: SequenceSet,
         mailbox: Mailbox,
         uid: bool,
     },
@@ -1379,7 +1454,9 @@ pub enum CommandBody {
     Idle,
 
     /// ----- Enable Extension (https://tools.ietf.org/html/rfc5161)
-    Enable { capabilities: Vec<Capability> },
+    Enable {
+        capabilities: NonEmptyVec<Capability>,
+    },
 
     /// ----- Compress Extension (https://tools.ietf.org/html/rfc4978) -----
     Compress { algorithm: CompressionAlgorithm },
@@ -1438,6 +1515,7 @@ impl Encode for CommandBody {
 
                 if let Some(ir) = initial_response {
                     writer.write_all(b" ")?;
+                    let ir = b64encode(ir);
                     writer.write_all(ir.as_bytes())?;
                 };
 
@@ -1510,13 +1588,16 @@ impl Encode for CommandBody {
                 writer.write_all(b" ")?;
                 mailbox_wildcard.encode(writer)
             }
-            CommandBody::Status { mailbox, items } => {
+            CommandBody::Status {
+                mailbox,
+                attributes,
+            } => {
                 writer.write_all(b"STATUS")?;
                 writer.write_all(b" ")?;
                 mailbox.encode(writer)?;
                 writer.write_all(b" ")?;
                 writer.write_all(b"(")?;
-                join_serializable(items, b" ", writer)?;
+                join_serializable(attributes, b" ", writer)?;
                 writer.write_all(b")")
             }
             CommandBody::Append {
@@ -1571,7 +1652,7 @@ impl Encode for CommandBody {
             }
             CommandBody::Fetch {
                 sequence_set,
-                items,
+                attributes,
                 uid,
             } => {
                 if *uid {
@@ -1580,9 +1661,9 @@ impl Encode for CommandBody {
                     writer.write_all(b"FETCH ")?;
                 }
 
-                join_serializable(sequence_set, b",", writer)?;
+                sequence_set.encode(writer)?;
                 writer.write_all(b" ")?;
-                items.encode(writer)
+                attributes.encode(writer)
             }
             CommandBody::Store {
                 sequence_set,
@@ -1597,7 +1678,7 @@ impl Encode for CommandBody {
                     writer.write_all(b"STORE ")?;
                 }
 
-                join_serializable(sequence_set, b",", writer)?;
+                sequence_set.encode(writer)?;
                 writer.write_all(b" ")?;
 
                 match kind {
@@ -1627,7 +1708,7 @@ impl Encode for CommandBody {
                 } else {
                     writer.write_all(b"COPY ")?;
                 }
-                join_serializable(sequence_set, b",", writer)?;
+                sequence_set.encode(writer)?;
                 writer.write_all(b" ")?;
                 mailbox.encode(writer)
             }
@@ -1645,9 +1726,10 @@ impl Encode for CommandBody {
 }
 
 /// The currently defined status data items that can be requested.
+#[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
 #[cfg_attr(feature = "serdex", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub enum StatusItem {
+pub enum StatusAttribute {
     /// The number of messages in the mailbox.
     Messages,
 
@@ -1664,14 +1746,14 @@ pub enum StatusItem {
     Unseen,
 }
 
-impl Encode for StatusItem {
+impl Encode for StatusAttribute {
     fn encode(&self, writer: &mut impl Write) -> std::io::Result<()> {
         match self {
-            StatusItem::Messages => writer.write_all(b"MESSAGES"),
-            StatusItem::Recent => writer.write_all(b"RECENT"),
-            StatusItem::UidNext => writer.write_all(b"UIDNEXT"),
-            StatusItem::UidValidity => writer.write_all(b"UIDVALIDITY"),
-            StatusItem::Unseen => writer.write_all(b"UNSEEN"),
+            StatusAttribute::Messages => writer.write_all(b"MESSAGES"),
+            StatusAttribute::Recent => writer.write_all(b"RECENT"),
+            StatusAttribute::UidNext => writer.write_all(b"UIDNEXT"),
+            StatusAttribute::UidValidity => writer.write_all(b"UIDVALIDITY"),
+            StatusAttribute::Unseen => writer.write_all(b"UNSEEN"),
         }
     }
 }
@@ -1690,11 +1772,11 @@ pub enum SearchKey {
     //     and multiple search keys.
     //
     // See also the corresponding `search` parser.
-    And(Vec<SearchKey>),
+    And(NonEmptyVec<SearchKey>),
 
     /// Messages with message sequence numbers corresponding to the
     /// specified message sequence number set.
-    SequenceSet(Vec<Sequence>),
+    SequenceSet(SequenceSet),
 
     /// All messages in the mailbox; the default initial key for ANDing.
     All,
@@ -1708,7 +1790,7 @@ pub enum SearchKey {
 
     /// Messages whose internal date (disregarding time and timezone)
     /// is earlier than the specified date.
-    Before(NaiveDate),
+    Before(MyNaiveDate),
 
     /// Messages that contain the specified string in the body of the
     /// message.
@@ -1760,7 +1842,7 @@ pub enum SearchKey {
 
     /// Messages whose internal date (disregarding time and timezone)
     /// is within the specified date.
-    On(NaiveDate),
+    On(MyNaiveDate),
 
     /// Messages that match either search key.
     Or(Box<SearchKey>, Box<SearchKey>), // TODO: is this a Vec or a single SearchKey?
@@ -1773,19 +1855,19 @@ pub enum SearchKey {
 
     /// Messages whose [RFC-2822] Date: header (disregarding time and
     /// timezone) is earlier than the specified date.
-    SentBefore(NaiveDate),
+    SentBefore(MyNaiveDate),
 
     /// Messages whose [RFC-2822] Date: header (disregarding time and
     /// timezone) is within the specified date.
-    SentOn(NaiveDate),
+    SentOn(MyNaiveDate),
 
     /// Messages whose [RFC-2822] Date: header (disregarding time and
     /// timezone) is within or later than the specified date.
-    SentSince(NaiveDate),
+    SentSince(MyNaiveDate),
 
     /// Messages whose internal date (disregarding time and timezone)
     /// is within or later than the specified date.
-    Since(NaiveDate),
+    Since(MyNaiveDate),
 
     /// Messages with an [RFC-2822] size smaller than the specified
     /// number of octets.
@@ -1805,7 +1887,7 @@ pub enum SearchKey {
 
     /// Messages with unique identifiers corresponding to the specified
     /// unique identifier set.  Sequence set ranges are permitted.
-    Uid(Vec<Sequence>),
+    Uid(SequenceSet),
 
     /// Messages that do not have the \Answered flag set.
     Unanswered,
@@ -1922,10 +2004,10 @@ impl Encode for SearchKey {
             SearchKey::Smaller(number) => write!(writer, "SMALLER {}", number),
             SearchKey::Uid(sequence_set) => {
                 writer.write_all(b"UID ")?;
-                join_serializable(sequence_set, b",", writer)
+                sequence_set.encode(writer)
             }
             SearchKey::Undraft => writer.write_all(b"UNDRAFT"),
-            SearchKey::SequenceSet(sequence_set) => join_serializable(sequence_set, b",", writer),
+            SearchKey::SequenceSet(sequence_set) => sequence_set.encode(writer),
             SearchKey::And(search_keys) => {
                 writer.write_all(b"(")?;
                 join_serializable(search_keys, b" ", writer)?;
@@ -1937,20 +2019,22 @@ impl Encode for SearchKey {
 
 #[cfg(test)]
 mod test {
+    use std::convert::TryInto;
+
+    use chrono::DateTime;
+
     use crate::{
         codec::Encode,
         types::{
-            command::{Command, SearchKey, StatusItem},
+            command::{Command, SearchKey, StatusAttribute},
             core::{AString, IString},
-            data_items::{DataItem, Macro, Part, Section},
+            datetime::MyDateTime,
+            fetch_attributes::{FetchAttribute, Macro, Part, Section},
             flag::{Flag, StoreResponse, StoreType},
-            mailbox::{ListMailbox, Mailbox},
-            sequence::ToSequence,
+            mailbox::ListMailbox,
             AuthMechanism,
         },
     };
-    use chrono::{SubsecRound, Utc};
-    use std::convert::TryInto;
 
     #[test]
     fn test_command_new() {
@@ -1961,85 +2045,106 @@ mod test {
             Command::starttls(),
             Command::authenticate(AuthMechanism::Plain, None),
             Command::authenticate(AuthMechanism::Login, None),
-            Command::authenticate(AuthMechanism::Plain, Some("XXXXXXXX")),
-            Command::authenticate(AuthMechanism::Login, Some("YYYYYYYY")),
-            Command::login("alice", "I_am_an_atom"),
-            Command::login("alice", "I am \\ \"quoted\""),
-            Command::login("alice", "I am a literal²"),
+            Command::authenticate(AuthMechanism::Plain, Some(b"XXXXXXXX")),
+            Command::authenticate(AuthMechanism::Login, Some(b"YYYYYYYY")),
+            Command::login("alice", "I_am_an_atom").unwrap(),
+            Command::login("alice", "I am \\ \"quoted\"").unwrap(),
+            Command::login("alice", "I am a literal²").unwrap(),
             Command::login(
-                AString::Atom("alice".into()),
-                AString::String(crate::types::core::IString::Literal(vec![0xff, 0xff, 0xff])),
-            ),
-            Command::select(Mailbox::Inbox),
-            Command::select(Mailbox::Other("atom".into())),
-            Command::select(Mailbox::Other("C:\\".into())),
-            Command::select(Mailbox::Other("²".into())),
-            Command::select(Mailbox::Other("Trash".into())),
-            Command::examine(Mailbox::Inbox),
-            Command::examine(Mailbox::Other("atom".into())),
-            Command::examine(Mailbox::Other("C:\\".into())),
-            Command::examine(Mailbox::Other("²".into())),
-            Command::examine(Mailbox::Other("Trash".into())),
-            Command::create(Mailbox::Inbox),
-            Command::delete(Mailbox::Inbox),
-            Command::rename(Mailbox::Inbox, Mailbox::Inbox),
-            Command::subscribe(Mailbox::Inbox),
-            Command::unsubscribe(Mailbox::Inbox),
-            Command::list(Mailbox::Inbox, "test"),
-            Command::list(Mailbox::Inbox, ListMailbox::Token("test".into())),
+                AString::Atom("alice".try_into().unwrap()),
+                AString::String(crate::types::core::IString::Literal(
+                    vec![0xff, 0xff, 0xff].try_into().unwrap(),
+                )),
+            )
+            .unwrap(),
+            Command::select("inbox").unwrap(),
+            Command::select("atom").unwrap(),
+            Command::select("C:\\").unwrap(),
+            Command::select("²").unwrap(),
+            Command::select("Trash").unwrap(),
+            Command::examine("inbox").unwrap(),
+            Command::examine("atom").unwrap(),
+            Command::examine("C:\\").unwrap(),
+            Command::examine("²").unwrap(),
+            Command::examine("Trash").unwrap(),
+            Command::create("inBoX").unwrap(),
+            Command::delete("inBOX").unwrap(),
+            Command::rename("iNBoS", "INboX").unwrap(),
+            Command::subscribe("inbox").unwrap(),
+            Command::unsubscribe("INBOX").unwrap(),
+            Command::list("iNbOx", "test").unwrap(),
+            Command::list("inbox", ListMailbox::Token("test".try_into().unwrap())).unwrap(),
             Command::lsub(
-                Mailbox::Inbox,
-                ListMailbox::String(IString::Quoted("\x7f".into())),
-            ),
-            Command::list("inBoX", ListMailbox::Token("test".into())),
-            Command::lsub("INBOX", ListMailbox::String(IString::Quoted("\x7f".into()))),
-            Command::status("inbox", vec![StatusItem::Messages]),
+                "inbox",
+                ListMailbox::String(IString::Quoted("\x7f".try_into().unwrap())),
+            )
+            .unwrap(),
+            Command::list("inBoX", ListMailbox::Token("test".try_into().unwrap())).unwrap(),
+            Command::lsub(
+                "INBOX",
+                ListMailbox::String(IString::Quoted("\x7f".try_into().unwrap())),
+            )
+            .unwrap(),
+            Command::status("inbox", vec![StatusAttribute::Messages]).unwrap(),
             Command::append(
                 "inbox",
                 vec![],
-                Some(Utc::now().trunc_subsecs(0).into()),
+                Some(MyDateTime(
+                    DateTime::parse_from_rfc2822("Tue, 1 Jul 2003 10:52:37 +0200").unwrap(),
+                )),
                 vec![0xff, 0xff, 0xff],
-            ),
+            )
+            .unwrap(),
             Command::append(
                 "inbox",
                 vec![Flag::Keyword("test".try_into().unwrap())],
-                Some(Utc::now().trunc_subsecs(0).into()),
+                Some(MyDateTime(
+                    DateTime::parse_from_rfc2822("Tue, 1 Jul 2003 10:52:37 +0200").unwrap(),
+                )),
                 vec![0xff, 0xff, 0xff],
-            ),
+            )
+            .unwrap(),
             Command::check(),
             Command::close(),
             Command::expunge(),
             Command::search(
                 None,
-                SearchKey::And(vec![SearchKey::All, SearchKey::New, SearchKey::Unseen]),
+                SearchKey::And(
+                    vec![SearchKey::All, SearchKey::New, SearchKey::Unseen]
+                        .try_into()
+                        .unwrap(),
+                ),
                 false,
-            ),
+            )
+            .unwrap(),
             Command::search(
                 None,
-                SearchKey::And(vec![SearchKey::All, SearchKey::New, SearchKey::Unseen]),
+                SearchKey::And(
+                    vec![SearchKey::All, SearchKey::New, SearchKey::Unseen]
+                        .try_into()
+                        .unwrap(),
+                ),
                 true,
-            ),
+            )
+            .unwrap(),
             //Command::search(None, SearchKey::And(vec![SearchKey::SequenceSet(vec![Sequence::Single(SeqNo::Value(42))])]), true),
-            Command::search(
-                None,
-                SearchKey::SequenceSet("42".to_sequence().unwrap()),
-                true,
-            ),
-            Command::search(
-                None,
-                SearchKey::SequenceSet("*".to_sequence().unwrap()),
-                true,
-            ),
+            Command::search(None, SearchKey::SequenceSet("42".try_into().unwrap()), true).unwrap(),
+            Command::search(None, SearchKey::SequenceSet("*".try_into().unwrap()), true).unwrap(),
             Command::search(
                 None,
                 SearchKey::Or(Box::new(SearchKey::Draft), Box::new(SearchKey::All)),
                 true,
-            ),
+            )
+            .unwrap(),
             Command::fetch(
                 "1",
-                vec![DataItem::BodyExt {
+                vec![FetchAttribute::BodyExt {
                     partial: None,
-                    section: Some(Section::Part(Part(vec![1, 1]))), // TODO: Part must be non-zero.
+                    section: Some(Section::Part(Part(
+                        vec![1.try_into().unwrap(), 1.try_into().unwrap()]
+                            .try_into()
+                            .unwrap(),
+                    ))),
                     peek: true,
                 }],
                 false,
