@@ -1,22 +1,26 @@
 //! # 7. Server Responses
 
+use std::{convert::TryInto, io::Write, num::NonZeroU32};
+
+#[cfg(feature = "arbitrary")]
+use arbitrary::Arbitrary;
+#[cfg(feature = "serdex")]
+use serde::{Deserialize, Serialize};
+
 use crate::{
     codec::Encode,
     types::{
         body::BodyStructure,
         core::{Atom, Charset, NString, Tag, Text},
-        data_items::Section,
+        datetime::MyDateTime,
         envelope::Envelope,
+        fetch_attributes::Section,
         flag::{Flag, FlagNameAttribute},
         mailbox::Mailbox,
         AuthMechanism, CompressionAlgorithm,
     },
     utils::{escape_quoted, join, join_serializable},
 };
-use chrono::{DateTime, FixedOffset};
-#[cfg(feature = "serdex")]
-use serde::{Deserialize, Serialize};
-use std::{convert::TryInto, io::Write};
 
 /// Server responses are in three forms.
 #[cfg_attr(feature = "serdex", derive(Serialize, Deserialize))]
@@ -346,7 +350,7 @@ pub enum Data {
         /// Name
         mailbox: Mailbox,
         /// Status parenthesized list
-        items: Vec<StatusItemResponse>,
+        attributes: Vec<StatusAttributeValue>,
     },
 
     /// ### 7.2.5. SEARCH Response
@@ -358,7 +362,7 @@ pub enum Data {
     /// search criteria.  For SEARCH, these are message sequence numbers;
     /// for UID SEARCH, these are unique identifiers.  Each number is
     /// delimited by a space.
-    Search(Vec<u32>),
+    Search(Vec<NonZeroU32>),
 
     /// ### 7.2.6.  FLAGS Response
     ///
@@ -454,7 +458,7 @@ pub enum Data {
     ///   response MAY be sent during a UID command.
     ///
     /// The update from the EXPUNGE response MUST be recorded by the client.
-    Expunge(u32),
+    Expunge(NonZeroU32),
 
     /// ### 7.4.2. FETCH Response
     ///
@@ -465,9 +469,9 @@ pub enum Data {
     /// flag updates).
     Fetch {
         /// Message SEQ or UID
-        seq_or_uid: u32,
+        seq_or_uid: NonZeroU32,
         /// Message data
-        items: Vec<DataItemResponse>,
+        attributes: Vec<MessageAttribute>,
     },
 
     /// ----- ENABLE Extension (RFC 5161) -----
@@ -517,11 +521,14 @@ impl Encode for Data {
                 writer.write_all(b" ")?;
                 mailbox.encode(writer)?;
             }
-            Data::Status { mailbox, items } => {
+            Data::Status {
+                mailbox,
+                attributes,
+            } => {
                 writer.write_all(b"* STATUS ")?;
                 mailbox.encode(writer)?;
                 writer.write_all(b" (")?;
-                join_serializable(items, b" ", writer)?;
+                join_serializable(attributes, b" ", writer)?;
                 writer.write_all(b")")?;
             }
             Data::Search(seqs) => {
@@ -540,9 +547,12 @@ impl Encode for Data {
             Data::Exists(count) => write!(writer, "* {} EXISTS", count)?,
             Data::Recent(count) => write!(writer, "* {} RECENT", count)?,
             Data::Expunge(msg) => write!(writer, "* {} EXPUNGE", msg)?,
-            Data::Fetch { seq_or_uid, items } => {
+            Data::Fetch {
+                seq_or_uid,
+                attributes,
+            } => {
                 write!(writer, "* {} FETCH (", seq_or_uid)?;
-                join_serializable(items, b" ", writer)?;
+                join_serializable(attributes, b" ", writer)?;
                 writer.write_all(b")")?;
             }
             Data::Enabled { capabilities } => {
@@ -558,7 +568,7 @@ impl Encode for Data {
 /// The currently defined status data items.
 #[cfg_attr(feature = "serdex", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub enum StatusItemResponse {
+pub enum StatusAttributeValue {
     /// The number of messages in the mailbox.
     Messages(u32),
 
@@ -567,17 +577,17 @@ pub enum StatusItemResponse {
 
     /// The next unique identifier value of the mailbox.  Refer to
     /// section 2.3.1.1 for more information.
-    UidNext(u32),
+    UidNext(NonZeroU32),
 
     /// The unique identifier validity value of the mailbox.  Refer to
     /// section 2.3.1.1 for more information.
-    UidValidity(u32),
+    UidValidity(NonZeroU32),
 
     /// The number of messages which do not have the \Seen flag set.
     Unseen(u32),
 }
 
-impl std::fmt::Display for StatusItemResponse {
+impl std::fmt::Display for StatusAttributeValue {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
         match self {
             Self::Messages(count) => write!(f, "MESSAGES {}", count),
@@ -589,7 +599,7 @@ impl std::fmt::Display for StatusItemResponse {
     }
 }
 
-impl Encode for StatusItemResponse {
+impl Encode for StatusAttributeValue {
     fn encode(&self, writer: &mut impl Write) -> std::io::Result<()> {
         write!(writer, "{}", self)
     }
@@ -727,19 +737,19 @@ pub enum Code {
     /// Followed by a decimal number, indicates the next unique
     /// identifier value.  Refer to section 2.3.1.1 for more
     /// information.
-    UidNext(u32),
+    UidNext(NonZeroU32),
 
     /// `UIDVALIDITY`
     ///
     /// Followed by a decimal number, indicates the unique identifier
     /// validity value.  Refer to section 2.3.1.1 for more information.
-    UidValidity(u32),
+    UidValidity(NonZeroU32),
 
     /// `UNSEEN`
     ///
     /// Followed by a decimal number, indicates the number of the first
     /// message without the \Seen flag set.
-    Unseen(u32),
+    Unseen(NonZeroU32),
 
     /// Additional response codes defined by particular client or server
     /// implementations SHOULD be prefixed with an "X" until they are
@@ -797,6 +807,7 @@ impl Encode for Code {
     }
 }
 
+#[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
 #[cfg_attr(feature = "serdex", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum Capability {
@@ -854,7 +865,7 @@ impl Encode for Capability {
 /// The current data items are:
 #[cfg_attr(feature = "serdex", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub enum DataItemResponse {
+pub enum MessageAttribute {
     /// A form of BODYSTRUCTURE without extension data.
     ///
     /// `BODY`
@@ -920,7 +931,7 @@ pub enum DataItemResponse {
     /// A string representing the internal date of the message.
     ///
     /// `INTERNALDATE`
-    InternalDate(DateTime<FixedOffset>),
+    InternalDate(MyDateTime),
 
     /// Equivalent to BODY[].
     ///
@@ -949,12 +960,12 @@ pub enum DataItemResponse {
     /// A number expressing the unique identifier of the message.
     ///
     /// `UID`
-    Uid(u32),
+    Uid(NonZeroU32),
 }
 
-impl Encode for DataItemResponse {
+impl Encode for MessageAttribute {
     fn encode(&self, writer: &mut impl Write) -> std::io::Result<()> {
-        use DataItemResponse::*;
+        use MessageAttribute::*;
 
         match self {
             BodyExt {
@@ -1015,9 +1026,10 @@ impl Encode for DataItemResponse {
 
 #[cfg(test)]
 mod test {
-    use super::*;
     use std::convert::TryFrom;
 
+    use super::*;
+
     #[test]
     fn test_status() {
         let tests: Vec<(_, &[u8])> = vec![
@@ -1111,14 +1123,22 @@ mod test {
                 Data::List {
                     items: vec![FlagNameAttribute::Noselect],
                     delimiter: Some('/'),
-                    mailbox: "bbb".into(),
+                    mailbox: "bbb".try_into().unwrap(),
                 },
                 b"* LIST (\\Noselect) \"/\" bbb\r\n",
             ),
-            (Data::Search(vec![1, 2, 3, 42]), b"* SEARCH 1 2 3 42\r\n"),
+            (
+                Data::Search(vec![
+                    1.try_into().unwrap(),
+                    2.try_into().unwrap(),
+                    3.try_into().unwrap(),
+                    42.try_into().unwrap(),
+                ]),
+                b"* SEARCH 1 2 3 42\r\n",
+            ),
             (Data::Exists(42), b"* 42 EXISTS\r\n"),
             (Data::Recent(12345), b"* 12345 RECENT\r\n"),
-            (Data::Expunge(123), b"* 123 EXPUNGE\r\n"),
+            (Data::Expunge(123.try_into().unwrap()), b"* 123 EXPUNGE\r\n"),
         ];
 
         for (parsed, serialized) in tests.into_iter() {
